/*
 * State.h
 *
 *  Created on: 21.07.2009
 *      Author: ewert
 */

#ifndef STATE_H_
#define STATE_H_

#include <string>
#include <list>
#include <map>
#include "Object.h"

namespace CCU {

class State {
private:
	std::map<std::string,Object> objects;
public:
	State();
	virtual ~State();


	/**
	 * Renames an Object
	 * \param oldObject name of the Object to be renamed
	 * \param newObject new name
	 * \return true iff the name has been changed
	 */
	bool renameObject(std::string oldObject, std::string newObject);

	/**
	 * Allows to add an Object to the state.
	 * Unique name assumption!
	 * \param object name/identifier of the object to add
	 * \return true iff the object has been added
	 */
	bool addObject(std::string object);

	/**
	 * Allows to add an Property to an object of the state
	 * \param object name/identifier of the object that receives the property
	 * \param property the name of the property
	 * \return true iff the property has been added
	 */
	bool addProperty(std::string object, std::string property);

	/**
	 * Allows to add an Relation between two objects of the state
	 * \param from name/identifier of the object from which the relation starts
	 * \param to name/identifier of the object in which the relation ends
	 * \param property the name of the property
	 * \return true iff the property has been added
	 */
	bool addRelation(std::string from, std::string to, std::string relation);

	/**
	 * Returns true iff the object is already conatined in the state
	 * \param object name/identifier of the object
	 */
	bool containsObject(std::string object)const;

	/**
	 * Returns a list of strings containing all names of the objects present in the state
	 * \return list is empty if no objects present
	 */
	std::list<std::string> getObjects() const;

	/**
	 * Returns all names of objects that have the given property
	 * \param property name of the property that is to be queried
	 * \return list of strings, empty iff there is no object with that given property
	 */
	std::list<std::string> getObjectsWithProperty(std::string property) const;

	/**
	 * Returns all objects that have a starting relationship with the given name
	 * \param relation
	 * \return list of strings, empty iff there is no object with that given relation
	 */
	std::list<std::string> getObjectsWithRelation(std::string relation) const;
	/**
	 * Returns a list of strings containing all properties of the object queried for
	 * \param object id of the object whose properties are queried
	 * \return list is empty iff no object with the queried name exists or no properties exist
	 */
	std::list<std::string> getPropertiesOf(std::string object) const;

	/**
	 * Returns a list of strings containing the names of the relations starting from the given object
	 * \param object id of the object whose relations are queried
	 * \return list is empty iff no object with the queried name exists or no properties exist
	 */
	std::list<std::string> getRelationsFrom(std::string object) const;

	/**
	 *  Returns a list containing all objects that have a relation that starts from the given object
	 */
	std::list<std::string> getRelatedObjects(std::string object) const;

	/**
	 *  Returns a list containing all objects that have the given relation that starts from the given object
	 *
	 */
	std::list<std::string> getRelatedObjects(std::string object,
			std::string relation) const ;

	/**
	 * Returns a string description of the state-object
	 * \return string
	 */
	std::string toString() const;

	// ********************************
	// lenoch - 20.08 - 14.10
	/**
	 * 14.10 -
	 * Swichted from + and - operator to "addState" and "removeState"
	 * (always had some problems with the operators, so i decided not to
	 * use them)
	 */
	bool addState(const State &);
	bool removeState(const State &);

	/**
	 * Returns the relation type from an object to another
	 * \param the object where the relations starts
	 * \param the object where the relation ends
	 * \return the relation type
	 */
	// std::list<std::string> getRelationsToObject(std::string from,std::string to) const;


	/*
	 * I cant stand the fact, that I'm not able to delete Properties, Relations
	 * and Objects :-/ (Ich weiß ja warum!! ... aber das erhöht den Arbeitsaufwand
	 * und damit die Fehlerquellen um vermutlich 1500%)
	 */
	bool removeObject(const std::string object);
	bool removeRelation(const std::string from, const std::string to, const std::string type);
	bool removeProperty(const std::string object, const std::string property);
};

}

#endif /* STATE_H_ */
